@@ -205,14 +205,8 @@ bool OSDMap::containing_subtree_is_down(CephContext *cct, int id, int subtree_ty
     down_cache = &local_down_cache;
   }
 
-  if (!subtree_is_down(id, down_cache)) {
-    ldout(cct, 30) << "containing_subtree_is_down(" << id << ") = false" << dendl;
-    return false;
-  }
-
   int current = id;
   while (true) {
-    // invariant: current subtree is known to be down.
     int type;
     if (current >= 0) {
       type = 0;
@@ -221,6 +215,11 @@ bool OSDMap::containing_subtree_is_down(CephContext *cct, int id, int subtree_ty
     }
     assert(type >= 0);
 
+    if (!subtree_is_down(current, down_cache)) {
+      ldout(cct, 30) << "containing_subtree_is_down(" << id << ") = false" << dendl;
+      return false;
+    }
+
     // is this a big enough subtree to be done?
     if (type >= subtree_type) {
       ldout(cct, 30) << "containing_subtree_is_down(" << id << ") = true ... " << type << " >= " << subtree_type << dendl;
@@ -454,11 +453,18 @@ void OSDMap::Incremental::dump(Formatter *f) const
   for (map<int64_t,pg_pool_t>::const_iterator p = new_pools.begin(); p != new_pools.end(); ++p) {
     f->open_object_section("pool");
     f->dump_int("pool", p->first);
-    f->dump_string("name", new_pool_names.find(p->first)->second);
     p->second.dump(f);
     f->close_section();
   }
   f->close_section();
+  f->open_array_section("new_pool_names");
+  for (map<int64_t,string>::const_iterator p = new_pool_names.begin(); p != new_pool_names.end(); ++p) {
+    f->open_object_section("pool_name");
+    f->dump_int("pool", p->first);
+    f->dump_string("name", p->second);
+    f->close_section();
+  }
+  f->close_section();
   f->open_array_section("old_pools");
   for (set<int64_t>::const_iterator p = old_pools.begin(); p != old_pools.end(); ++p)
     f->dump_int("pool", *p);
